package com.loafle.overflow.crawler.snmp;

import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.crawler.config.Config;
import com.loafle.overflow.crawler.config.Connection;
import com.loafle.overflow.crawler.config.Item;

import com.loafle.overflow.crawler.result.OFResultSet;
import com.loafle.overflow.crawler.snmp.version.SNMPv2c;
import com.loafle.overflow.crawler.snmp.version.SNMPv3;
import org.snmp4j.Snmp;
import org.snmp4j.TransportMapping;
import org.snmp4j.smi.Address;
import org.snmp4j.transport.DefaultUdpTransportMapping;

import java.io.IOException;
import java.util.*;

/**
 * Created by insanity on 17. 4. 10.
 */
public class SNMPCrawler extends Crawler {

    @Override
    public Object getInternal(Config config) {


//        String targetVer = (String)config.getTarget().getAuth().get("version");
//        switch (targetVer) {
//            case "v2c":
//                retObj = getByV2c(config);
//                break;
//            case "v3":
//                retObj = getByV3(config);
//                break;
//            default:
//                new Exception("Unknown SNMP protocol : " + targetVer).printStackTrace();
//        }

        return getSNMP(config);
    }

    private List<OFResultSet> getSNMP(Config config) {

        Snmp snmp = null;
        TransportMapping<? extends Address> transport = null;

        List<OFResultSet> rslist = null;

        try {
            transport = new DefaultUdpTransportMapping();
            snmp = new Snmp(transport);
            transport.listen();

            rslist = new ArrayList<>();

            List<Item> items = config.getItems();
            String[] oids = null;

            Map<String, String> snmpResultMap = null;
            for (Item item : items) {
                oids = new String[item.getKeys().size()];
                for (int i=0; i<item.getKeys().size();i++) {
                    oids[i] = item.getKeys().get(i).getKey();
                }

                snmpResultMap = getSNMPV(snmp, config, oids);
                rslist.add(convertResultSet(snmpResultMap, item));

            }
            return rslist;

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (snmp != null) {
                try {
                    snmp.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (transport != null) {
                try {
                    transport.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }


        return rslist;
    }

    private Map<String, String> getSNMPV(Snmp snmp, Config config, String[] oids) throws Exception {

        String targetVer = (String)config.getTarget().getAuth().get("version");
        switch (targetVer) {
            case "v2c":
                return getV2c(snmp, config, oids);
            case "v3":
                return getV3(snmp, config, oids);
            default:
                new Exception("Unknown SNMP protocol : " + targetVer).printStackTrace();
        }


        return null;
    }

    private Map<String, String> getV2c(Snmp snmp, Config config, String[] oids) throws Exception {

        SNMPv2c snmpV2 = new SNMPv2c(snmp);

        Map<String, Object> authInfo = config.getTarget().getAuth();
        Connection connInfo = config.getTarget().getConnection();

        String ip = connInfo.getIp();
        String port = connInfo.getPort();
        String community = (String)authInfo.get("community");

        return snmpV2.get(ip, port, community, oids);
    }

    private Map<String, String> getV3(Snmp snmp, Config config, String[] oids) throws Exception {
        SNMPv3 snmpV3 = new SNMPv3(snmp);

        Map<String, Object> authInfo = config.getTarget().getAuth();
        Connection connInfo = config.getTarget().getConnection();

        String ip = connInfo.getIp();
        String port = connInfo.getPort();
//            String method = config.get;

        String user = (String)authInfo.get("user");
        String authType = (String)authInfo.get("authType");
        String authPass = (String)authInfo.get("authPass");
        String privType = (String)authInfo.get("privType");
        String privPass = (String)authInfo.get("privPass");

        return snmpV3.get(ip, port, user, authType, authPass, privType, privPass, oids);
    }

    protected OFResultSet convertResultSet(Map<String, String> map, Item item) {

        OFResultSet rs = OFResultSet.newInstance(item);

        List<String> row = new ArrayList<>(Arrays.asList(new String[map.size()]));

        Map<String, Integer> metaMap = rs.getMeta();

        for (Map.Entry<String, Integer> info : metaMap.entrySet()) {
            row.set(info.getValue().intValue(), map.get(info.getKey()));
        }
        rs.addRow(row);

        return rs;
    }
//
//    private List<OFResultSet> getByV2c(Config config) {
//        Snmp snmp = null;
//        TransportMapping<? extends Address> transport = null;
//
//        List<OFResultSet> rslist = null;
//
//        try {
//            transport = new DefaultUdpTransportMapping();
//            snmp = new Snmp(transport);
//            transport.listen();
//            SNMPv2c snmpV2 = new SNMPv2c(snmp);
//
//            rslist = new ArrayList<>();
//
//            Map<String, Object> authInfo = config.getTarget().getAuth();
//            Connection connInfo = config.getTarget().getConnection();
//
//            String ip = connInfo.getIp();
//            String port = connInfo.getPort();
//            String community = (String)authInfo.get("community");
//
//            List<Item> items = config.getItems();
//            String[] oids = null;
//            for (Item item : items) {
//                oids = new String[item.getKeys().size()];
////                List<Query> queries = item.getQueries();
//                for (int i=0; i<item.getKeys().size();i++) {
//                    oids[i] = item.getKeys().get(i).getKey();
//                }
//
//                rslist.add(convertResultSet(snmpV2.get(ip, port, community, oids), item));
//
//            }
//            return rslist;
//            /*
//            switch (method) {
//                case "validate":
//                    return snmpV2.validate(ip, port, community);
//                case "get":
//                    return snmpV2.get(ip, port, community, oids);
//                case "walk":
//                    return snmpV2.walk(ip, port, community, oids[0]);
//                default:
//                    new Exception("Unknown SNMP command : " + method).printStackTrace();
//            }
//            */
//
//        } catch (Exception e) {
//            e.printStackTrace();
//        } finally {
//            if (snmp != null) {
//                try {
//                    snmp.close();
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//            }
//            if (transport != null) {
//                try {
//                    transport.close();
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//            }
//        }
//        return null;
//    }
//
//    private Object getByV3(Config config) {
//        Snmp snmp = null;
//        TransportMapping<? extends Address> transport = null;
//
//        List<OFResultSet> rslist = null;
//
//        try {
//            transport = new DefaultUdpTransportMapping();
//            snmp = new Snmp(transport);
//            transport.listen();
//            SNMPv3 snmpV3 = new SNMPv3(snmp);
//
//            rslist = new ArrayList<>();
//
//            Map<String, Object> authInfo = config.getTarget().getAuth();
//            Connection connInfo = config.getTarget().getConnection();
//
//            String ip = connInfo.getIp();
//            String port = connInfo.getPort();
////            String method = config.get;
//
//            String user = (String)authInfo.get("user");
//            String authType = (String)authInfo.get("authType");
//            String authPass = (String)authInfo.get("authPass");
//            String privType = (String)authInfo.get("privType");
//            String privPass = (String)authInfo.get("privPass");
//
//            List<Item> items = config.getItems();
//            String[] oids = null;
//            for (Item item : items) {
//                oids = new String[item.getKeys().size()];
////                List<Query> queries = item.getQueries();
//                for (int i=0; i<item.getKeys().size();i++) {
//                    oids[i] = item.getKeys().get(i).getKey();
//                }
//
////                rslist.add(convertResultSet(snmpV2.get(ip, port, community, oids), item));
//                rslist.add(convertResultSet(snmpV3.get(ip, port, user, authType, authPass, privType, privPass, oids), item));
//            }
//
//            return rslist;
//
//            /*
//            switch (method) {
//                case "validate":
//                    return snmpV3.validate(ip, port, user, authType, authPass, privType, privPass);
//                case "get":
//                    return snmpV3.get(ip, port, user, authType, authPass, privType, privPass, oids);
//                case "walk":
//                    return snmpV3.walk(ip, port, user, authType, authPass, privType, privPass, oids[0]);
//                default:
//                    new Exception("Unknown SNMP command : " + method).printStackTrace();
//            }
//            */
//
//        } catch (Exception e) {
//            new Exception("An Exception happend : " + e.getMessage()).printStackTrace();
//        } finally {
//            if (snmp != null) {
//                try {
//                    snmp.close();
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//            }
//            if (transport != null) {
//                try {
//                    transport.close();
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//            }
//        }
//        return null;
//    }



}