package com.loafle.overflow.crawler.snmp.version;

import org.snmp4j.PDU;
import org.snmp4j.Snmp;
import org.snmp4j.Target;
import org.snmp4j.event.ResponseEvent;
import org.snmp4j.smi.OID;
import org.snmp4j.smi.VariableBinding;
import org.snmp4j.util.TreeEvent;
import org.snmp4j.util.TreeUtils;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by insanity on 17. 4. 10.
 */
public abstract class SNMP {

    protected Map<String, String> get(Snmp snmp, PDU pdu, Target target) throws Exception {

        ResponseEvent response = snmp.get(pdu, target);

        if (response == null) {
            throw new Exception("Error: No response from SNMP Agent.");
        }

        PDU responsePDU = response.getResponse();
        if (responsePDU == null) {
            throw new Exception("Error: Response PDU is null");
        }

        int errorStatus = responsePDU.getErrorStatus();
        int errorIndex = responsePDU.getErrorIndex();
        String errorStatusText = responsePDU.getErrorStatusText();

        if (errorStatus != PDU.noError) {
            throw new Exception("Error: " + errorIndex + " " + errorStatusText);
        }

        Map<String, String> result = new LinkedHashMap<String, String>();
        for (VariableBinding varBinding : responsePDU.getVariableBindings()) {
            if (varBinding == null) {
                continue;
            }
            result.put(varBinding.getOid().toString(), varBinding.getVariable().toString());
        }
        return result;
    }

    protected Map<String, String> walk(String oidStr, Target target, TreeUtils treeUtils) throws Exception {
        OID oid = new OID(oidStr);
        List<TreeEvent> events = treeUtils.getSubtree(target, oid);

        if(events == null || events.size() == 0){
            throw new Exception("No data.");
        }

        Map<String, String> result = new LinkedHashMap<String, String>();
        for (TreeEvent event : events) {
            if(event == null) {
                continue;
            }
            if (event.isError()) {
                System.err.println(event.getErrorMessage());
                continue;
            }

            VariableBinding[] varBindings = event.getVariableBindings();

            for (VariableBinding varBinding : varBindings) {
                if (varBinding == null) {
                    continue;
                }
                result.put(varBinding.getOid().toString(), varBinding.getVariable().toString());
            }
        }
        return result;
    }
}
