package com.loafle.overflow.crawler.snmp.version;

import org.snmp4j.*;
import org.snmp4j.mp.MPv3;
import org.snmp4j.mp.MessageProcessingModel;
import org.snmp4j.mp.SnmpConstants;
import org.snmp4j.security.*;
import org.snmp4j.smi.*;
import org.snmp4j.transport.DefaultUdpTransportMapping;
import org.snmp4j.util.PDUFactory;
import org.snmp4j.util.TreeUtils;

import java.util.Map;

/**
 * Created by insanity on 17. 4. 10.
 */
public class SNMPv3 extends SNMP {

    private Snmp snmp =  null;

    public SNMPv3(Snmp snmp) {
        this.snmp = snmp;
    }

    public Boolean validate(String addr, String port, String user, String authType, String authPass, String privType, String privPass) throws Exception {
        String testOid = "1.3.6.1.2.1.1.3.0";
        Map<String, String> result = this.get(addr, port, user, authType, authPass, privType, privPass, new String[]{testOid});
        if (result.get(testOid) != null) return true;
        return false;
    }

    public Map<String, String> walk(String addr, String port, String user, String authType, String authPass, String privType, String privPass, String oidStr)
            throws Exception {
        USM usm = new USM(SecurityProtocols.getInstance(), new OctetString(MPv3.createLocalEngineID()), 0);
        SecurityModels.getInstance().addSecurityModel(usm);

        UsmUser usmUser = getUser(user, authType, authPass, privType, privPass);
        this.snmp.getUSM().addUser(new OctetString(user), usmUser);


        TreeUtils treeUtils = new TreeUtils(this.snmp, new PDUFactory() {
            public PDU createPDU(Target target) {
                ScopedPDU sp = new ScopedPDU();
                sp.setRequestID(new Integer32(1));
                return sp;
            }

            public PDU createPDU(MessageProcessingModel messageProcessingModel) {
                return new ScopedPDU();
            }
        });

        Target target = getTarget(addr, port, usmUser);
        return super.walk(oidStr, target, treeUtils);
    }

    public Map<String, String> get(String addr, String port, String user, String authType, String authPass, String privType, String privPass, String[] oids)
            throws Exception {

        TransportMapping<? extends Address> transport = new DefaultUdpTransportMapping();
        Snmp snmp = new Snmp(transport);
        transport.listen();

        USM usm = new USM(SecurityProtocols.getInstance(), new OctetString(MPv3.createLocalEngineID()), 0);
        SecurityModels.getInstance().addSecurityModel(usm);

        UsmUser usmUser = getUser(user, authType, authPass, privType, privPass);
        this.snmp.getUSM().addUser(new OctetString(user), usmUser);

        PDU pdu = new ScopedPDU();
        for (String oid : oids) {
            pdu.addOID(new VariableBinding(new OID(oid)));
        }
        pdu.setType(PDU.GET);
        pdu.setRequestID(new Integer32(1));

        Target target = getTarget(addr, port, usmUser);

        return super.get(snmp, pdu, target);
    }

    private UserTarget getTarget(String addr, String port, UsmUser user) {

        UdpAddress address = new UdpAddress(addr + "/" + port);
        UserTarget target = new UserTarget();
        target.setAddress(address);
        target.setRetries(2);
        target.setTimeout(5000);
        target.setVersion(SnmpConstants.version3);
        target.setSecurityModel(user.getSecurityModel());

        int secLv;
        OID autyType = user.getAuthenticationProtocol();
        OID privType = user.getPrivacyProtocol();
        if (autyType != null && privType != null) {
            secLv = SecurityLevel.AUTH_PRIV;
        }else if (autyType != null && privType == null) {
            secLv = SecurityLevel.AUTH_NOPRIV;
        }else {
            secLv = SecurityLevel.NOAUTH_NOPRIV;
        }
        target.setSecurityLevel(secLv);
        target.setSecurityName(new OctetString(user.getSecurityName()));
        return target;
    }

    private UsmUser getUser(String user, String authType, String authPass, String privType, String privPass) {
        OctetString octetUser = new OctetString(user);
        OctetString octetAuthPass = authPass.equals("") ? null : new OctetString(authPass);
        OctetString octetPrivPass = privPass.equals("") ? null :  new OctetString(privPass);

        UsmUser usmUser =  new UsmUser(octetUser, getAuthOID(authType), octetAuthPass, getPrivOID(privType), octetPrivPass);
        return usmUser;
    }

    private OID getAuthOID(String authType) {
        OID authOID = null;
        switch(authType) {
            case "":
                authOID = null;
                break;
            case "MD5":
                authOID = AuthMD5.ID;
                break;
            case "SHA":
                authOID = AuthSHA.ID;
                break;
        }
        return authOID;
    }

    private OID getPrivOID(String privType) {
        OID privOID = null;
        switch(privType) {
            case "":
                privOID = null;
                break;
            case "DES":
                privOID = PrivDES.ID;
                break;
            case "AES":
                privOID = PrivAES128.ID;
                break;
        }
        return privOID;
    }
}
